/*
 *  Copyright (C) 2006 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>

#include <pspctrl.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <psppower.h>
#include <pspiofilemgr.h>

#include "psp_global.h"
#include "psp_sdl.h"
#include "psp_menu.h"
#include "psp_fmgr.h"
#include "psp_battery.h"
#include "psp_menu_help.h"
#include "psp_kbd.h"
#include "psp_go.h"

extern SDL_Surface *back_surface;
static int psp_menu_dirty = 1;

# define MENU_GAME_MODE    0
# define MENU_BOARD_SIZE   1
# define MENU_PLAYER_COLOR 2
# define MENU_HANDICAP     3
# define MENU_KOMI         4
# define MENU_LEVEL        5
# define MENU_CLOCK        6
# define MENU_SCREENSHOT   7

# define MENU_HELP         8

# define MENU_NEW_GAME     9
# define MENU_RESIGN_GAME 10
# define MENU_LOAD_SGF    11
# define MENU_SAVE_SGF    12
# define MENU_BACK        13
# define MENU_SAVE_CFG    14
# define MENU_EXIT        15

# define MAX_MENU_ITEM (MENU_EXIT + 1)

  static menu_item_t menu_list[] =
  { 
    { "Game mode       : " },
    { "Board size      : " },
    { "Player color    : " },
    { "Handicap        : " },
    { "Komi            : " },
    { "Level           : " },
    { "Clock frequency : " },
    { "Save Screenshot : " },

    { "Help" },

    { "New game" },
    { "Resign game" },
    { "Load SGF" },
    { "Save SGF" },
    { "Back to game" },
    { "Save config" },
    { "Exit" },
  };

  static int cur_menu_id = MENU_BACK;

  static int go_board_size   = 19;
  static int go_player_color = GGO_BLACK;
  static int go_handicap     = 0;
  static int go_level        = 10;
  static int go_komi         = 0;
  static int go_game_mode    = GGO_HUMAN_VS_COMPUTER;
  static int psp_cpu_clock   = 222;

void
string_fill_with_space(char *buffer, int size)
{
  int length = strlen(buffer);
  int index;

  for (index = length; index < size; index++) {
    buffer[index] = ' ';
  }
  buffer[size] = 0;
}

void
psp_display_screen_menu_battery(void)
{
  char buffer[64];

  int Length;
  int color;

  snprintf(buffer, 50, " [%s] ", psp_get_battery_string());
  Length = strlen(buffer);

  if (psp_is_low_battery()) color = PSP_MENU_RED_COLOR;
  else                      color = PSP_MENU_GREEN_COLOR;

  psp_sdl_back2_print(240 - ((8*Length) / 2), 6, buffer, color);
}

static void 
psp_display_screen_menu(void)
{
  char buffer[32];
  int menu_id = 0;
  int color   = 0;
  int x       = 0;
  int y       = 0;
  int y_step  = 0;

  //if (psp_menu_dirty) 
  {
    psp_sdl_blit_background();
    psp_menu_dirty = 0;

    psp_sdl_draw_rectangle(10,10,459,249,PSP_MENU_BORDER_COLOR,0);
    psp_sdl_draw_rectangle(11,11,457,247,PSP_MENU_BORDER_COLOR,0);

    psp_sdl_back2_print( 30, 6, " Start+L+R: EXIT ", PSP_MENU_WARNING_COLOR);

    psp_display_screen_menu_battery();

    psp_sdl_back2_print( 360, 6, " R: New Game ", PSP_MENU_NOTE_COLOR);

    psp_sdl_back2_print(30, 254, " []: Cancel  O/X: Valid  Select: Back ", 
                       PSP_MENU_BORDER_COLOR);

    psp_sdl_back2_print(370, 254, " By Zx-81 ",
                       PSP_MENU_AUTHOR_COLOR);
  }
  
  x      = 20;
  y      = 25;
  y_step = 10;
  
  for (menu_id = 0; menu_id < MAX_MENU_ITEM; menu_id++) {
    color = PSP_MENU_TEXT_COLOR;
    if (cur_menu_id == menu_id) color = PSP_MENU_SEL_COLOR;
    else 
    if (menu_id == MENU_EXIT) color = PSP_MENU_RED_COLOR;
    else
    if (menu_id == MENU_HELP) color = PSP_MENU_GREEN_COLOR;

    psp_sdl_back2_print(x, y, menu_list[menu_id].title, color);

    if (menu_id == MENU_BOARD_SIZE) {
      sprintf(buffer,"%d", go_board_size);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_PLAYER_COLOR) {
      if (go_player_color == GGO_BLACK) strcpy(buffer, "black");
      else                              strcpy(buffer, "white");
      string_fill_with_space(buffer, 5);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_HANDICAP) {
      sprintf(buffer,"%d", go_handicap);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_KOMI) {
      sprintf(buffer,"%.1f", ((float)go_komi) / 10.0);
      string_fill_with_space(buffer, 6);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_GAME_MODE) {
      if (go_game_mode == GGO_HUMAN_VS_COMPUTER) strcpy(buffer, "Human vs PSP");
      else
      if (go_game_mode == GGO_HUMAN_VS_HUMAN) strcpy(buffer, "Human vs Human");
      else                                    strcpy(buffer, "PSP vs PSP");
      string_fill_with_space(buffer, 15);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_LEVEL) {
      sprintf(buffer,"%d", go_level);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_CLOCK) {
      sprintf(buffer,"%d", psp_cpu_clock);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_SCREENSHOT) {
      sprintf(buffer,"%d", GGO.psp_screenshot_id);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(170, y, buffer, color);
      y += y_step;
    } else
    if (menu_id == MENU_SAVE_CFG) {
      y += y_step;
    } else
    if (menu_id == MENU_HELP) {
      y += y_step;
    } else
    if (menu_id == MENU_EXIT) {
      y += y_step;
    }


    y += y_step;
  }
}

void
psp_main_menu_new_game(void)
{
  psp_go_new_game();
}

void
psp_main_menu_resign_game(void)
{
  psp_go_resign_game();
}

void
psp_main_menu_load_sgf(void)
{
  int ret;

  ret = psp_fmgr_menu_load(FMGR_FORMAT_SGF);
  if (ret ==  1) /* load OK */
  {
    psp_menu_dirty = 1;
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  60, "File loaded !", PSP_MENU_NOTE_COLOR);
    psp_menu_dirty = 1;
    psp_sdl_flip();
    sleep(1);

    psp_go_set_move_mode(0);
  }
  else 
  if (ret == -1) /* Load Error */
  {
    psp_menu_dirty = 1;
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  60, "Can't load file !", PSP_MENU_WARNING_COLOR);
    psp_menu_dirty = 1;
    psp_sdl_flip();
    sleep(1);
  }

  go_board_size = GGO.board_width;
  go_handicap   = GGO.go_handicap;
}

static void
psp_main_menu_save_sgf()
{
  int error;
  error = psp_fmgr_menu_save();

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  60, "File saved !", PSP_MENU_NOTE_COLOR);
    psp_menu_dirty = 1;
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270,  60, "Can't save file !", PSP_MENU_WARNING_COLOR);
    psp_menu_dirty = 1;
    psp_sdl_flip();
    sleep(1);
  }
}

static void
psp_main_menu_validate(void)
{
  int need_to_reset = 0;
  GGO.psp_cpu_clock = psp_cpu_clock;
 
  if (go_handicap != GGO.go_handicap) {
    GGO.go_handicap = go_handicap;
  }
  if (go_komi != GGO.go_komi) {
    GGO.go_komi = go_komi;
  }
  if (go_game_mode != GGO.go_game_mode) {
    GGO.go_game_mode = go_game_mode;
  }
  if (go_level != GGO.go_level) {
    GGO.go_level = go_level;
  }
  if (go_player_color != GGO.go_player_color) {
    GGO.go_player_color = go_player_color;
    need_to_reset = 1;
  }
  if (go_board_size != GGO.board_width) {
    GGO.board_width = go_board_size;
    need_to_reset = 1;
  }

  if (need_to_reset) {
    psp_main_menu_new_game();
  }
  scePowerSetClockFrequency(GGO.psp_cpu_clock, GGO.psp_cpu_clock, GGO.psp_cpu_clock/2);
}


static void
psp_main_menu_save_config()
{
  int error;

  psp_main_menu_validate();
  error = psp_save_configuration();

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270, 60, "File saved !", PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270, 60, "Can't save file !", PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

static void
psp_main_menu_screenshot(void)
{
  GGO.psp_screenshot_mode = 1;
}

int
psp_main_menu_exit(void)
{
  SceCtrlData c;

  psp_display_screen_menu();
  psp_sdl_back2_print(270,  60, "press X to confirm !", PSP_MENU_WARNING_COLOR);
  psp_menu_dirty = 1;
  psp_sdl_flip();

  psp_kbd_wait_no_button();

  do
  {
    sceCtrlReadBufferPositive(&c, 1);
    c.Buttons &= PSP_ALL_BUTTON_MASK;

    if (c.Buttons & PSP_CTRL_CROSS) psp_sdl_exit(0);

  } while (c.Buttons == 0);

  psp_kbd_wait_no_button();

  return 0;
}

static void
psp_main_menu_init(void)
{
  psp_cpu_clock    = GGO.psp_cpu_clock;
  go_handicap      = GGO.go_handicap;
  go_komi          = GGO.go_komi;
  go_game_mode     = GGO.go_game_mode;
  go_level         = GGO.go_level;
  go_player_color  = GGO.go_player_color;
  go_board_size    = GGO.board_width;
}

void
psp_main_menu_board_size( int step )
{
  if (step > 0) {
    if (go_board_size < 19) go_board_size++;
    else                    go_board_size = 5;
  } else {
    if (go_board_size > 5) go_board_size--;
    else                   go_board_size = 19;
  }
}

void
psp_main_menu_player_color( int step )
{
  if (go_player_color == GGO_BLACK) go_player_color = GGO_WHITE;
  else                              go_player_color = GGO_BLACK;
}

void
psp_main_menu_game_mode( int step )
{
  if (step > 0) {
    if (go_game_mode < GGO_COMPUTER_VS_COMPUTER) go_game_mode++;
    else                                         go_game_mode = 0;
  } else {
    if (go_game_mode > 0) go_game_mode--;
    else                  go_game_mode = GGO_COMPUTER_VS_COMPUTER;
  }
}

void
psp_main_menu_handicap( int step )
{
  if (step > 0) {
    if (go_handicap < 9) go_handicap++;
    else                 go_handicap = 0;
  } else {
    if (go_handicap > 0) go_handicap--;
    else                 go_handicap = 9;
  }
}

void
psp_main_menu_komi( int step )
{
  if (step > 0) {
    if (go_komi < 100) go_komi += 5;
    else               go_komi  = 0;
  } else {
    if (go_komi >  0)  go_komi -= 5;
    else               go_komi  = 100;
  }
}

void
psp_main_menu_level( int step )
{
  if (step > 0) {
    if (go_level < 10) go_level++;
    else               go_level = 0;
  } else {
    if (go_level > 0) go_level--;
    else              go_level = 10;
  }
}

#define MAX_CLOCK_VALUES 4
static int clock_values[MAX_CLOCK_VALUES] = { 222, 266, 300, 333 };

static void
psp_main_menu_clock( int step )
{
  int index;
  for (index = 0; index < MAX_CLOCK_VALUES; index++) {
    if (psp_cpu_clock == clock_values[index]) break;
  }
  if (step > 0) {
    index++;
    if (index >= MAX_CLOCK_VALUES) index = 0;
    psp_cpu_clock = clock_values[index];

  } else {
    index--;

    if (index < 0) index = MAX_CLOCK_VALUES - 1;
    psp_cpu_clock = clock_values[index];
  }
}


int 
psp_main_menu(void)
{
  SceCtrlData c;
  long        new_pad;
  long        old_pad;
  int         last_time;
  int         end_menu;

  psp_kbd_wait_no_button();

  old_pad   = 0;
  last_time = 0;
  end_menu  = 0;

  psp_main_menu_init();

  psp_menu_dirty = 1;

  while (! end_menu)
  {
    psp_display_screen_menu();

    psp_sdl_flip();

    while (1)
    {
      sceCtrlReadBufferPositive(&c, 1);
      c.Buttons &= PSP_ALL_BUTTON_MASK;

      if (c.Buttons) break;
    }

    new_pad = c.Buttons;

    if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > PSP_MENU_MIN_TIME)) {
      last_time = c.TimeStamp;
      old_pad = new_pad;

    } else continue;

    if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) ==
        (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) {
      /* Exit ! */
      psp_sdl_exit(0);
    } else
    if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_SELECT)) == (PSP_CTRL_LTRIGGER|PSP_CTRL_SELECT)) {
      /* Ir shell ? */
      sceKernelDelayThread(100);
    } else
    if ((c.Buttons & PSP_CTRL_RTRIGGER) == PSP_CTRL_RTRIGGER) {
      psp_main_menu_new_game();
      end_menu = 1;
    } else
    if ((new_pad & PSP_CTRL_CROSS ) || 
        (new_pad & PSP_CTRL_CIRCLE) || 
        (new_pad & PSP_CTRL_RIGHT ) ||
        (new_pad & PSP_CTRL_LEFT  )) 
    {
      int step;

      if (new_pad & PSP_CTRL_LEFT)  step = -1;
      else 
      if (new_pad & PSP_CTRL_RIGHT) step =  1;
      else                          step =  0;

      switch (cur_menu_id ) 
      {
        case MENU_BOARD_SIZE   : psp_main_menu_board_size( step );
        break;
        case MENU_PLAYER_COLOR : psp_main_menu_player_color( step );
        break;
        case MENU_HANDICAP     : psp_main_menu_handicap( step );
        break;
        case MENU_LEVEL        : psp_main_menu_level( step );
        break;
        case MENU_KOMI         : psp_main_menu_komi( step );
        break;
        case MENU_GAME_MODE    : psp_main_menu_game_mode( step );
        break;
        case MENU_CLOCK        : psp_main_menu_clock( step );
        break;
        case MENU_SCREENSHOT   : psp_main_menu_screenshot();
                                 end_menu = 1;
        break;              

        case MENU_NEW_GAME     : psp_main_menu_new_game();
                                 end_menu = 1;
        break;                 
        case MENU_RESIGN_GAME  : psp_main_menu_resign_game();
                                 end_menu = 1;
        break;                 
        case MENU_LOAD_SGF     : psp_main_menu_load_sgf();
        break;                 
        case MENU_SAVE_SGF     : psp_main_menu_save_sgf();
        break;                 
        case MENU_SAVE_CFG     : psp_main_menu_save_config();
        break;               
        case MENU_BACK         : end_menu = 1;
        break;                 
        case MENU_EXIT         : psp_main_menu_exit();
        break;                 
        case MENU_HELP         : psp_menu_dirty = 1;
                                 psp_help_menu();
                                 old_pad = new_pad = 0;
                                 psp_menu_dirty = 1;
        break;              
      }

    } else
    if(new_pad & PSP_CTRL_UP) {

      if (cur_menu_id > 0) cur_menu_id--;
      else                 cur_menu_id = MAX_MENU_ITEM-1;

    } else
    if(new_pad & PSP_CTRL_DOWN) {

      if (cur_menu_id < (MAX_MENU_ITEM-1)) cur_menu_id++;
      else                                 cur_menu_id = 0;

    } else  
    if(new_pad & PSP_CTRL_SQUARE) {
      /* Cancel */
      end_menu = -1;
    } else 
    if((new_pad & PSP_CTRL_SELECT) == PSP_CTRL_SELECT) {
      /* Back to CPC */
      end_menu = 1;
    }
  }

  if (end_menu > 0) {
    psp_main_menu_validate();
  }
 
  psp_kbd_wait_no_button();

  psp_sdl_clear_screen( PSP_MENU_BLACK_COLOR );
  psp_sdl_flip();
  psp_sdl_clear_screen( PSP_MENU_BLACK_COLOR );
  psp_sdl_flip();

  return 1;
}

