/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <pspwlan.h>
#include <pspkernel.h>
#include <psppower.h>
#include <pspctrl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "psp_nlh.h"
#include "psp_sdl.h"
#include "psp_menu.h"

#define MAX_PICK 5
#define MAX_PICK_TITLE 80
#define MAX_PICK_MAINSTR 13
#define MAX_PICK_FINEPRINT 45

typedef struct
{
    char szBig[MAX_PICK_MAINSTR+1]; // big font
    char szFinePrint[MAX_PICK_FINEPRINT+1]; // small font
    u32 userData;
} PICKER_INFO;

typedef struct
{
    char szTitle[MAX_PICK_TITLE+1];
    int pick_count;
    PICKER_INFO picks[MAX_PICK];
    int pick_start; // -1 for none, 0->pick_count-1 for selection
} PICKER;

void 
my_initpicker(PICKER* pickerP, const char* szTitle)
{
  strncpy(pickerP->szTitle, szTitle, MAX_PICK_TITLE);
  pickerP->szTitle[MAX_PICK_TITLE] = '\0';
  pickerP->pick_count = 0;
  pickerP->pick_start = -1;
}

int 
my_addpick(PICKER* pickerP, const char* szBig, const char* szFinePrint, u32 userData)
{
    PICKER_INFO* pickP;
    if (pickerP->pick_count >= MAX_PICK)
        return 0; // no room
        // REVIEW: in future provide a smaller font version with more slots
    pickP = &(pickerP->picks[pickerP->pick_count++]);

    strncpy(pickP->szBig, szBig, MAX_PICK_MAINSTR);
    pickP->szBig[MAX_PICK_MAINSTR] = '\0';
    if (szFinePrint == NULL)
        szFinePrint = "";
    strncpy(pickP->szFinePrint, szFinePrint, MAX_PICK_FINEPRINT);
    pickP->szFinePrint[MAX_PICK_FINEPRINT] = '\0';
    pickP->userData = userData;
    return 1;
}

int 
my_loadpicks_fromfile(PICKER* pickerP, const char* szFile, int bParseIP)
    // returns number of entries read
{
    FILE* pf = fopen(szFile, "rt");
    if (pf == 0)
        return 0;
    char szLine[128];
    int nAdded = 0;
    while (fgets(szLine, sizeof(szLine), pf) != NULL)
    {
        // CR-LF will cause blank lines
        if (szLine[0] == '\0' || szLine[0] == ';')
            continue;   // skip

        // format is "BIG_STRING;FinePrint"
        char* szFinePrint = strchr(szLine, ';');
        if (szFinePrint != NULL)
            *szFinePrint++ = '\0';
        u32 userData = 0;
        if (bParseIP)
        {
            // parse szLine for ?.?.?.? IP address
            userData = sceNetInetInetAddr(szLine);
            if (userData == 0)
                continue;   // bad, skip it
        }
        // only first 5 will fit
        if (my_addpick(pickerP, szLine, szFinePrint, userData))
            nAdded++;
    }
    fclose(pf);
    return nAdded;
}

/////////////////////////////////////////////////////
// Actual UI

#define Y_PICK(i)    (2+3*(i))
#define X_PICK       (4)
#define STR_PICK "=>"

void
my_psp_sdl_back2_print(int x, int y, char *text, int color)
{
  psp_sdl_back2_print(x * 8, y * 8, text, color);
}

int 
my_picker(const PICKER* pickerP)
{
    int iPick = pickerP->pick_start;
    u32 buttonsLast = 0;
    int bRedraw = 1;

    if (pickerP->pick_count == 0) {
        return -1; // nothing to pick
    }

    sceCtrlSetSamplingCycle(0);
    sceCtrlSetSamplingMode(0); // digital

    while (1)
    {
      int i;
      SceCtrlData pad;
      u32 buttonsNew;

# if 0 //LUDO:
      if (pickerP->pick_count == 1) {
        // auto-pick if only one
        iPick = 0;
        break;
      }
# endif

      if (bRedraw) {
        psp_sdl_blit_background();
        my_psp_sdl_back2_print(0,0, pickerP->szTitle, PSP_MENU_NOTE_COLOR);

        for (i = 0; i < pickerP->pick_count; i++)
        {
          PICKER_INFO const* pickP = &pickerP->picks[i];
          int text_color = PSP_MENU_TEXT_COLOR;
          if (i == iPick)
          {
            text_color = PSP_MENU_SEL_COLOR;
            my_psp_sdl_back2_print(0, Y_PICK(i), STR_PICK, text_color);
          }
          my_psp_sdl_back2_print(X_PICK, Y_PICK(i), pickP->szBig, text_color);
          my_psp_sdl_back2_print(X_PICK + 2, Y_PICK(i) + 1, pickP->szFinePrint, text_color);
        }
        bRedraw = 0;
      }

      // handle up/down inputs
      sceCtrlReadBufferPositive(&pad, 1); 
      buttonsNew = pad.Buttons & ~buttonsLast;
      buttonsLast = pad.Buttons;

      if (buttonsNew & PSP_CTRL_DOWN) {
        iPick++;
        if (iPick >= pickerP->pick_count)
            iPick = pickerP->pick_count-1;
        bRedraw = 1;
      }
      else if (buttonsNew & PSP_CTRL_UP) {
        iPick--;
        if (iPick < 0) iPick = 0;
        bRedraw = 1;
      }
      else if (buttonsNew & (PSP_CTRL_CIRCLE | PSP_CTRL_CROSS)) {
        if (iPick >= 0 && iPick < pickerP->pick_count) {
              break; // picked !
        }
      }
      else if (buttonsNew & PSP_CTRL_SQUARE)
      {
        iPick = -1; // cancel
        break;
      }
    }
    return iPick;
}



int 
psp_wifi_connect(int argc, char **argv)
{
  int info_thid;
  int last_state;
  u32 err;
  char szMyIPAddr[32];
  int connectionConfig = -1;
  PICKER pickConn; // connection picker
  int iNetIndex;
  int iPick;

  psp_sdl_blit_background();

  // nlhInit must be called from user thread for DHCP to work
  err = nlhInit();

  if (err != 0) {
    pspDebugScreenPrintf("nlhInit: %d !\n", err);
    nlhTerm();
    return;
  }

# if 0
  info_thid = sceKernelCreateThread( "info_thread", 
     (SceKernelThreadEntry)info_thread, 0x18, 0x10000, 0, 0 );

  if(info_thid >= 0) {
    sceKernelStartThread(info_thid, 0, 0);
  }
# endif

  while (sceWlanGetSwitchState() != 1)  //switch = off
  {
    my_psp_sdl_back2_print(10, 10, "Turn the wifi switch on!", PSP_MENU_TEXT_COLOR);
    sceKernelDelayThread(1000*1000);
  }

  // enumerate connections
  my_initpicker(&pickConn, "Choose a connection:                      X OK    [] CANCEL");

  for (iNetIndex = 1; iNetIndex < 100; iNetIndex++) // skip the 0th connection
  {
    char data[128];
    char name[128];
    char detail[128];
    if (sceUtilityCheckNetParam(iNetIndex) != 0) continue;
    sceUtilityGetNetParam(iNetIndex, 0, name);

    sceUtilityGetNetParam(iNetIndex, 1, data);
    strcpy(detail, "SSID=");
    strcat(detail, data);

    sceUtilityGetNetParam(iNetIndex, 4, data);
    if (data[0]) {
      // not DHCP
      sceUtilityGetNetParam(iNetIndex, 5, data);
      strcat(detail, " IPADDR=");
      strcat(detail, data);
    } else {
      strcat(detail, " DHCP");
    }

    my_addpick(&pickConn, name, detail, (u32)iNetIndex);
    if (pickConn.pick_count >= MAX_PICK) break;  // no more
  }

  if (pickConn.pick_count == 0) {
    my_psp_sdl_back2_print(10,10, "No wifi connections, please create one !", PSP_MENU_TEXT_COLOR);
    goto close_net;
  }

  pickConn.pick_start = 0;
  iPick = my_picker(&pickConn);
  if (iPick == -1) goto close_net;
  connectionConfig = (int)(pickConn.picks[iPick].userData);

connect_wifi:

  err = sceNetApctlConnect(connectionConfig);
  if (err != 0) {
    goto close_net;
  }

  {
    char buffer[128];
    psp_sdl_blit_background();
    strcpy(buffer, "Trying to connect to ");
    strcat(buffer, pickConn.picks[iPick].szBig);
    my_psp_sdl_back2_print(0,0, buffer, PSP_MENU_TEXT_COLOR);
  }

  // 4 connected with IP address
  
  last_state = -1;

  while (1) 
  {
    int state = 0;
    char buffer[128];

    sceKernelDelayThread(50*1000);
    err = sceNetApctlGetState(&state);

    if (err != 0)
    {
      my_psp_sdl_back2_print(0,1, "Connection failed !", PSP_MENU_TEXT_COLOR);
      goto close_connection;
    }

    {
      SceCtrlData c;
      sceCtrlReadBufferPositive(&c, 1);
      if (c.Buttons & PSP_CTRL_SQUARE) goto close_connection;
    }

    if (state != last_state) 
    {
      if (last_state == 2 && state == 0)
      {
        my_psp_sdl_back2_print(0,2, "Connecting to wifi Failed, Retrying...", PSP_MENU_TEXT_COLOR);
        sceKernelDelayThread(500*1000); // 500ms
        goto connect_wifi;
      }
     
      sprintf(buffer, "Connection state %d of 4", state);
      my_psp_sdl_back2_print(0,1, buffer, PSP_MENU_TEXT_COLOR);
      last_state = state;
    }

    // 0 - idle
    // 1,2 - starting up
    // 3 - waiting for dynamic IP
    // 4 - got IP - usable
    if (state == 4) break;  // connected with static IP
  }

  // get IP address
  if (sceNetApctlGetInfo(8, szMyIPAddr) != 0) {
    strcpy(szMyIPAddr, "unknown IP address");
  }

# if 0  //LUDO:
  cli_main(argc, argv);
# endif

close_connection:
  err = sceNetApctlDisconnect();

close_net:
  psp_sdl_blit_background();
  my_psp_sdl_back2_print(0,0, "Closing connection ...", PSP_MENU_TEXT_COLOR);
  
  nlhTerm();
  sceKernelDelayThread(1000000); 

  sceKernelExitDeleteThread(info_thid);
  sceKernelWaitThreadEnd(info_thid, NULL);
}
